@@ -10,7 +10,7 @@ XML::ScannerStateEntry::ScannerStateEntry()
 {
 }
 
-XML::ScannerStateEntry::ScannerStateEntry(int state, ScannerStateType type)
+XML::ScannerStateEntry::ScannerStateEntry(unsigned state, ScannerStateType type)
 {
    _state = state;
    _type = type;
@@ -34,7 +34,7 @@ XML::ScannerStateType XML::ScannerStateEntry::getType()
 {
    return _type;
 }
-int XML::ScannerStateEntry::getState()
+unsigned XML::ScannerStateEntry::getState()
 {
    return _state;
 }
@@ -44,13 +44,13 @@ XML::ScannerTransEntry::ScannerTransEntry()
 {
 }
 
-XML::ScannerTransEntry::ScannerTransEntry(int state, char c)
+XML::ScannerTransEntry::ScannerTransEntry(unsigned state, char c)
 {
    _state = state;
    _c = c;
 }
 
-XML::ScannerTransEntry::ScannerTransEntry(int state, char c, int fstate)
+XML::ScannerTransEntry::ScannerTransEntry(unsigned state, char c, unsigned fstate)
 {
    _state = state;
    _c = c;
@@ -72,7 +72,7 @@ bool XML::ScannerTransEntry::operator == (const XML::ScannerTransEntry& s)
    return false;
 }
 
-int XML::ScannerTransEntry::getFState()
+unsigned XML::ScannerTransEntry::getFState()
 {
    return _fstate;
 }
@@ -150,7 +150,7 @@ XML::StackEntry::StackEntry()
    _symbol = NONE;
 }
 
-XML::StackEntry::StackEntry(Symbol symbol, int num, int state, Chain tval)
+XML::StackEntry::StackEntry(Symbol symbol, unsigned num, unsigned state, Chain tval)
 {
    _symbol=symbol;
    _num=num;
@@ -167,7 +167,7 @@ XML::StackEntry& XML::StackEntry::operator = (const XML::StackEntry& se)
    return(*this);
 }
 
-int XML::StackEntry::getState()
+unsigned XML::StackEntry::getState()
 {
    return _state;
 }
@@ -185,7 +185,7 @@ XML::ActionEntry::ActionEntry()
    _action=NULLACTION;
 }
 
-XML::ActionEntry::ActionEntry(Action action , int num)
+XML::ActionEntry::ActionEntry(Action action , unsigned num)
 {
    _action=action;
    _num=num;
@@ -210,7 +210,7 @@ XML::Action XML::ActionEntry::getAction()
    return _action;
 }
 
-int XML::ActionEntry::getNum()
+unsigned XML::ActionEntry::getNum()
 {
    return _num;
 }
@@ -219,12 +219,12 @@ XML::ProdEntry::ProdEntry()
 {
 }
 
-XML::ProdEntry::ProdEntry(int id)
+XML::ProdEntry::ProdEntry(unsigned id)
 {
    _id = id;
 }
 
-XML::ProdEntry::ProdEntry(int id, Production prod, int numSymbol)
+XML::ProdEntry::ProdEntry(unsigned id, Production prod, unsigned numSymbol)
 {
    _id = id;
    _prod = prod;
@@ -247,7 +247,7 @@ XML::ProdEntry& XML::ProdEntry::operator = (const XML::ProdEntry& pe)
 }
 
 
-int XML::ProdEntry::getId()
+unsigned XML::ProdEntry::getId()
 {
    return _id;
 }
@@ -257,7 +257,7 @@ XML::Production XML::ProdEntry::getProd()
    return _prod;
 }
 
-int XML::ProdEntry::getNumSymbol()
+unsigned XML::ProdEntry::getNumSymbol()
 {
    return _numSymbol;
 }
@@ -520,48 +520,45 @@ void XML::loadScanner()
 }
 void XML::loadParser0()
 {
-   _actionMap[58][LT] = ActionEntry(REDUCE, 22);
-   _actionMap[59][LT] = ActionEntry(REDUCE, 17);
-   _actionMap[59][ENDTOKEN] = ActionEntry(REDUCE, 17);
-   _actionMap[53][MT] = ActionEntry(SHIFT, 58);
-   _actionMap[54][LT] = ActionEntry(REDUCE, 13);
-   _actionMap[54][ENDTOKEN] = ActionEntry(REDUCE, 13);
-   _actionMap[55][LB] = ActionEntry(REDUCE, 24);
-   _actionMap[56][MT] = ActionEntry(SHIFT, 59);
-   _actionMap[57][QMARK] = ActionEntry(REDUCE, 4);
-   _actionMap[57][MT] = ActionEntry(REDUCE, 4);
-   _actionMap[57][IDENTIFIER] = ActionEntry(REDUCE, 4);
-   _actionMap[57][SLASH] = ActionEntry(REDUCE, 4);
-   _actionMap[45][RB] = ActionEntry(SHIFT, 53);
-   _actionMap[46][RB] = ActionEntry(REDUCE, 23);
-   _actionMap[47][MT] = ActionEntry(SHIFT, 54);
-   _actionMap[48][CDATA] = ActionEntry(SHIFT, 55);
-   _actionMap[49][IDENTIFIER] = ActionEntry(SHIFT, 56);
-   _actionMap[50][ENDTOKEN] = ActionEntry(REDUCE, 8);
-   _actionMap[50][LT] = ActionEntry(REDUCE, 8);
-   _actionMap[51][ENDTOKEN] = ActionEntry(REDUCE, 10);
-   _actionMap[51][LT] = ActionEntry(REDUCE, 10);
-   _actionMap[52][STRINGVAL] = ActionEntry(SHIFT, 57);
-   _actionMap[31][RB] = ActionEntry(SHIFT, 45);
-   _actionMap[32][DATA] = ActionEntry(SHIFT, 46);
-   _actionMap[33][IDENTIFIER] = ActionEntry(SHIFT, 47);
-   _actionMap[34][LB] = ActionEntry(SHIFT, 48);
+   _actionMap[56][LT] = ActionEntry(REDUCE, 17);
+   _actionMap[56][ENDTOKEN] = ActionEntry(REDUCE, 17);
+   _actionMap[52][LT] = ActionEntry(REDUCE, 13);
+   _actionMap[52][ENDTOKEN] = ActionEntry(REDUCE, 13);
+   _actionMap[53][LB] = ActionEntry(REDUCE, 24);
+   _actionMap[54][MT] = ActionEntry(SHIFT, 56);
+   _actionMap[55][QMARK] = ActionEntry(REDUCE, 4);
+   _actionMap[55][MT] = ActionEntry(REDUCE, 4);
+   _actionMap[55][IDENTIFIER] = ActionEntry(REDUCE, 4);
+   _actionMap[55][SLASH] = ActionEntry(REDUCE, 4);
+   _actionMap[45][LT] = ActionEntry(REDUCE, 23);
+   _actionMap[46][MT] = ActionEntry(SHIFT, 52);
+   _actionMap[47][CDATA] = ActionEntry(SHIFT, 53);
+   _actionMap[48][IDENTIFIER] = ActionEntry(SHIFT, 54);
+   _actionMap[49][ENDTOKEN] = ActionEntry(REDUCE, 8);
+   _actionMap[49][LT] = ActionEntry(REDUCE, 8);
+   _actionMap[50][ENDTOKEN] = ActionEntry(REDUCE, 10);
+   _actionMap[50][LT] = ActionEntry(REDUCE, 10);
+   _actionMap[51][STRINGVAL] = ActionEntry(SHIFT, 55);
+   _actionMap[31][LT] = ActionEntry(REDUCE, 22);
+   _actionMap[32][DATA] = ActionEntry(SHIFT, 45);
+   _actionMap[33][IDENTIFIER] = ActionEntry(SHIFT, 46);
+   _actionMap[34][LB] = ActionEntry(SHIFT, 47);
    _actionMap[35][LT] = ActionEntry(REDUCE, 18);
-   _actionMap[36][SLASH] = ActionEntry(SHIFT, 49);
+   _actionMap[36][SLASH] = ActionEntry(SHIFT, 48);
    _actionMap[36][IDENTIFIER] = ActionEntry(SHIFT, 17);
    _actionMap[36][STREAM] = ActionEntry(SHIFT, 16);
-   _actionMap[37][MT] = ActionEntry(SHIFT, 50);
+   _actionMap[37][MT] = ActionEntry(SHIFT, 49);
    _actionMap[38][MT] = ActionEntry(SHIFT, 39);
    _actionMap[38][SLASH] = ActionEntry(SHIFT, 40);
    _actionMap[38][IDENTIFIER] = ActionEntry(SHIFT, 42);
    _jumpMap[38][Attr] = 41;
    _actionMap[39][LT] = ActionEntry(REDUCE, 12);
-   _actionMap[40][MT] = ActionEntry(SHIFT, 51);
+   _actionMap[40][MT] = ActionEntry(SHIFT, 50);
    _actionMap[41][QMARK] = ActionEntry(REDUCE, 2);
    _actionMap[41][MT] = ActionEntry(REDUCE, 2);
    _actionMap[41][IDENTIFIER] = ActionEntry(REDUCE, 2);
    _actionMap[41][SLASH] = ActionEntry(REDUCE, 2);
-   _actionMap[42][EQU] = ActionEntry(SHIFT, 52);
+   _actionMap[42][EQU] = ActionEntry(SHIFT, 51);
    _actionMap[43][ENDTOKEN] = ActionEntry(REDUCE, 9);
    _actionMap[43][LT] = ActionEntry(REDUCE, 9);
    _actionMap[44][ENDTOKEN] = ActionEntry(REDUCE, 1);
@@ -570,12 +567,12 @@ void XML::loadParser0()
    _actionMap[20][LB] = ActionEntry(SHIFT, 32);
    _jumpMap[20][DataContent] = 31;
    _actionMap[21][SLASH] = ActionEntry(SHIFT, 33);
-}
-void XML::loadParser1()
-{
    _actionMap[21][EMARK] = ActionEntry(SHIFT, 34);
    _actionMap[22][LT] = ActionEntry(REDUCE, 14);
    _actionMap[22][ENDTOKEN] = ActionEntry(REDUCE, 14);
+}
+void XML::loadParser1()
+{
    _actionMap[23][LT] = ActionEntry(SHIFT, 36);
    _jumpMap[23][ElementPre] = 7;
    _jumpMap[23][Elem] = 35;
@@ -623,12 +620,12 @@ void XML::loadParser1()
    _jumpMap[4][XMLTag] = 4;
    _jumpMap[4][XMLBody] = 10;
    _actionMap[5][LT] = ActionEntry(REDUCE, 21);
-}
-void XML::loadParser2()
-{
    _jumpMap[5][DataList] = 12;
    _jumpMap[5][StreamContent] = 11;
    _actionMap[6][ENDTOKEN] = ActionEntry(REDUCE, 7);
+}
+void XML::loadParser2()
+{
    _actionMap[6][LT] = ActionEntry(REDUCE, 7);
    _actionMap[7][MT] = ActionEntry(SHIFT, 14);
    _jumpMap[7][ElementContent] = 13;
@@ -651,7 +648,7 @@ void XML::loadParser3()
 {
    _prodInfo.Insert(ProdEntry(24, DataPre, 4));
    _prodInfo.Insert(ProdEntry(23, DataContent, 2));
-   _prodInfo.Insert(ProdEntry(22, Data, 5));
+   _prodInfo.Insert(ProdEntry(22, Data, 2));
    _prodInfo.Insert(ProdEntry(21, DataList, 0));
    _prodInfo.Insert(ProdEntry(20, DataList, 2));
    _prodInfo.Insert(ProdEntry(19, ElementList, 0));
@@ -679,9 +676,9 @@ void XML::loadParser3()
 XML::XML()
 {
    loadScanner();
-   for ( int i=0; i < 60; i++)
+   for ( unsigned i=0; i < 57; i++)
    {
-      int j;
+      unsigned j;
       for ( j = 0; j < 18; j++)
          _actionMap[i][j] = ActionEntry();
       for ( j = 0; j < 18; j++)
@@ -712,7 +709,7 @@ void XML::parse()
    while ( true )
    {
       Action a;
-      int num;
+      unsigned num;
       if ( getAction(s.getTop()->getState(), _token, a, num) )
       {
          switch (a)
@@ -732,14 +729,14 @@ void XML::parse()
             {
                _tokenList.Empty();
                ProdEntry* pPE = _prodInfo.Find(num);
-               for (int i=0; i<pPE->getNumSymbol();i++)
+               for (unsigned i=0; i<pPE->getNumSymbol();i++)
                {
                   StackEntry se;
                   s.Pop(se);
                   if (se.getSymbol() == TOKEN)
                      _tokenList.Insert(se.getTval());
                }
-               int fstate;
+               unsigned fstate;
                if (getJump(s.getTop()->getState(), pPE->getProd(), fstate))
                {
                   StackEntry se(PROD, pPE->getId(), fstate, _tokenVal);
@@ -801,7 +798,7 @@ void XML::parse()
             {
                return;
             }
-            default:
+            case NULLACTION:
                break;
          }
       }
@@ -814,7 +811,7 @@ void XML::parse()
 }
 
 
-bool XML::getAction(int state, XML::Token token, XML::Action& a, int& num)
+bool XML::getAction(unsigned state, XML::Token token, XML::Action& a, unsigned& num)
 {
     if (_actionMap[state][token].getAction() != NULLACTION )
     {
@@ -828,7 +825,7 @@ bool XML::getAction(int state, XML::Token token, XML::Action& a, int& num)
     }
 }
 
-bool XML::getJump(int state, Production prod, int& fstate)
+bool XML::getJump(unsigned state, Production prod, unsigned& fstate)
 {
     if ( _jumpMap[state][prod] > 0 )
     {
@@ -888,6 +885,10 @@ bool XML::shiftToken()
       {
          _tokenVal[i]=c;
          i++;
+         if ( i >= MAXTOKENVAL )
+         {
+            throw Exception(EXLOC, Chain("Token length exceeded"));
+         }
          c = nextChar();
       }
       if (isSepSign(c)) backChar();
